
#include "Game.h"

boolean Game::Create(IShell * shell, IDisplay * display)
{
	mShell = shell;
	mDisplay = display;

	mDeviceInfo.wStructSize = sizeof(mDeviceInfo);
	ISHELL_GetDeviceInfo(mShell, &mDeviceInfo);

	DBGPRINTF(" *** Width %d, Height %d", GetWidth(), GetHeight());

	for (int i = 0; i < AVK_LAST - AVK_FIRST; i++)
	{
		mKeysDown[i] = FALSE;
	}

	mRenderer.Create(mShell, mDisplay);

	// Enable the zbuffer
	glEnable(GL_DEPTH_TEST);

	// Set the view port size to the window size
	glViewport(0, 0, GetWidth(), GetHeight());

	// Setup the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Diable lighting and alpha blending
	glDisable(GL_LIGHTING);
	glDisable(GL_BLEND);

	// Set the fustrum clipping planes
	glFrustumx(ITOFP(-5), ITOFP(5), ITOFP(-5), ITOFP(5), ITOFP(10),ITOFP(100));

	// Set the model view to identity
	glMatrixMode(GL_MODELVIEW );
	glLoadIdentity();

	// Enable the arrays we want used when we glDrawElements(..)
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);

	mRotateAngle = 0;

	return TRUE;
}

void Game::Destroy()
{
	mRenderer.Destroy();
}

void Game::Tick(int timeElapsed)
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glPushMatrix();
	glLoadIdentity();

	glTranslatex(0,0, ITOFP(-15));

	if (mKeysDown[AVK_LEFT - AVK_FIRST] == TRUE)
	{
		mRotateAngle -= 3;
	}
	if (mKeysDown[AVK_RIGHT - AVK_FIRST] == TRUE)
	{
		mRotateAngle += 3;
	}

	if (mRotateAngle < 0) mRotateAngle += 360;
	if (mRotateAngle > 360) mRotateAngle -= 360;

	glRotatex(ITOFP(mRotateAngle), ITOFP(0), ITOFP(1), ITOFP(0));

	int FaceData[9] =
	{
		-ITOFP(2), -ITOFP(2), ITOFP(0),			// First vertex position
		ITOFP(2),  -ITOFP(2), ITOFP(0),			// Second vertex position
		-ITOFP(0),  ITOFP(2), ITOFP(0)			// Third vertex position
	};

	int ColorData[12] =
	{
		ITOFP(1), ITOFP(0), ITOFP(0), ITOFP(0),	// First vertex color
		ITOFP(0), ITOFP(1), ITOFP(0), ITOFP(0),	// Second vertex color
		ITOFP(0), ITOFP(0), ITOFP(1), ITOFP(0)	// Third vertex color
	};

	uint8 IndexData[3] = {0, 1, 2};

	glVertexPointer(3, GL_FIXED, 0, FaceData);	// Set the vertex (position) data source
	glColorPointer(4, GL_FIXED, 0, ColorData);	// Set the color data source

	glDrawElements(GL_TRIANGLES, 3, GL_UNSIGNED_BYTE, IndexData);// Draw the triangle

	glPopMatrix();

	mRenderer.FlipToScreen();

	AECHAR buffer[16];
	WSPRINTF(buffer, 16, L"FPS: %d", 1000 / timeElapsed);
	IDISPLAY_DrawText(mDisplay, AEE_FONT_BOLD, buffer, -1, 5, 5, NULL, 0);

	IDISPLAY_Update(mDisplay);
}

void Game::KeyPressed(int keyCode)
{
	mKeysDown[keyCode - AVK_FIRST] = TRUE;
}

void Game::KeyReleased(int keyCode)
{
	mKeysDown[keyCode - AVK_FIRST] = FALSE;
}
