
#include "Game.h"

boolean Renderer::Create(IShell * shell, IDisplay * display)
{
	mIGL		= NULL;
	mIEGL		= NULL;

	mDisplay	= NULL;
	mConfig		= NULL;
	mSurface	= NULL;
	mContext	= NULL;

	if (ISHELL_CreateInstance(shell, AEECLSID_GL, (void **)&mIGL) != SUCCESS)
	{
		Destroy();
		return FALSE;
	}

	if (ISHELL_CreateInstance(shell, AEECLSID_EGL, (void **)&mIEGL) != SUCCESS)
	{
		Destroy();
		return FALSE;
	}

	IGL_Init(mIGL);
	IEGL_Init(mIEGL);

	mDisplay = eglGetDisplay(display);

	if (mDisplay == EGL_NO_DISPLAY ) 
	{
		DBGPRINTF(" *** eglGetDisplay failed");
		Destroy();
		return FALSE;
	}

	EGLint major = 0;
	EGLint minor = 0;

	if (eglInitialize(mDisplay, &major, &minor) == FALSE)
	{
		DBGPRINTF(" *** eglInitialize failed");
		Destroy();
		return FALSE;
	}

	DBGPRINTF(" *** ES version %d.%d", major, minor);

	EGLint numConfigs = 1;
	if (eglGetConfigs(mDisplay, &mConfig, 1, &numConfigs) == FALSE)
	{
		DBGPRINTF(" *** eglGetConfigs failed");
		Destroy();
		return false;
	}

	IBitmap *	DeviceBitmap = NULL; 
	IDIB *		DIB = NULL;

	if (IDISPLAY_GetDeviceBitmap(display, &DeviceBitmap) != SUCCESS) 
	{
		DBGPRINTF(" *** IDISPLAY_GetDeviceBitmap failed");
		Destroy();
		return FALSE;
	}

	if (IBITMAP_QueryInterface(DeviceBitmap, AEECLSID_DIB, (void**)&DIB) != SUCCESS) 
	{
		DBGPRINTF(" *** IBITMAP_QueryInterface failed");
		IBITMAP_Release(DeviceBitmap);
		Destroy();
		return FALSE;
	}

	EGLint params[5] = {EGL_NONE, EGL_NONE, EGL_NONE, EGL_NONE, EGL_NONE};
	mSurface = eglCreateWindowSurface(mDisplay, mConfig, DIB, params);

	IDIB_Release(DIB);
	IBITMAP_Release(DeviceBitmap);

	if (mSurface == EGL_NO_SURFACE) 
	{
		DBGPRINTF(" *** eglCreateWindowSurface failed");
		Destroy();
		return FALSE;
	}

	mContext = eglCreateContext(mDisplay, mConfig, NULL, NULL); 

	if (mContext == EGL_NO_CONTEXT)
	{
		DBGPRINTF(" *** eglCreateContext failed");
		Destroy();
		return FALSE;
	}

	if (eglMakeCurrent(mDisplay, mSurface, mSurface, mContext) == FALSE)
	{
		DBGPRINTF(" *** eglMakeCurrent failed");
		Destroy();
		return FALSE;
	}

	return TRUE;
}


void Renderer::Destroy()
{
	eglMakeCurrent(EGL_NO_DISPLAY, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);

	if (mContext)
	{
		eglDestroyContext(mDisplay, mContext);
		mContext = NULL;
	}

	if (mSurface)
	{
		eglDestroySurface(mDisplay, mSurface);
		mSurface = NULL;
	}

	if (mDisplay)
	{
		if (eglTerminate(mDisplay) == FALSE)
		{
			DBGPRINTF(" *** Could not eglTerminate(mGLDisplay)");
		}
		mDisplay = NULL;
	}

	if (mIEGL)
	{
		IEGL_Release(mIEGL);
		mIEGL = NULL;
	}

	if (mIGL)
	{
		IGL_Release(mIGL);
		mIGL = NULL;
	}
}

void Renderer::FlipToScreen()
{
	eglSwapBuffers(mDisplay, mSurface);
}
